;; Copyright 2009, 2010, 2012, 2013, 2014 Kevin Ryde

;; This file is part of weblint-el.
;;
;; weblint-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; weblint-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with weblint-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; xemacs21 has `line-number'
;; emacs22 has `line-number-at-pos'
;; emacs21 doesn't have this
(defun my-line-number (&optional pos)
  "Return the line number of point, or POS if given, in the current buffer."
  (save-excursion
    (if pos (goto-char pos))
    (beginning-of-line)
    (1+ (count-lines (point-min) (point)))))

(defun my-push (value symbol)
  "Prepend VALUE to a list in variable SYMBOL."
  (set symbol (cons value (symbol-value symbol))))

(defun my-compilation-next-error ()
  "Got to next error with `compilation-next-error', nil if no more.
If a next error is moved to then return the \"locus\" as per
`compilation-next-error'.  If not then return nil."

  (condition-case err
      (compilation-next-error 1)
    (error
     (if (member err '((error "Moved past last error")
                       ;; emacs24.3 up
                       (user-error "Moved past last error")))
         nil ;; return nil for no more errors
       ;; re-throw anything else
       (error "%s" (error-message-string err))))))

(defun my-compilation-error-lines ()
  "Return a list of matched error lines in a compilation-mode buffer.
This return is like
    ((LINENUM TARGET-FILENAME TARGET-LINENUM)
     ...
    )
The first LINENUM is the place in the current buffer, then the
TARGET parts are what the error match says.

FIXME: When there's two errors on the same line of a target file
only one of them is returned by the emacs21/xemacs21 code here."

  (require 'compile)
  (let (ret)
    (save-excursion
      (goto-char (point-min))

      (if (and (fboundp 'compilation-parse-errors)
               (not (fboundp 'compilation--compat-error-properties)))
	  (progn
            ;; emacs21, xemacs21

	    (or compilation-error-list
                (compilation-parse-errors (point-max) nil))
            (mapcar (lambda (err)
                      (let ((marker (car err))
                            (lst    (cdr err)))
                        (my-push (list (my-line-number (marker-position marker))
                                       (car (nth 0 lst))  ;; filename
                                       (nth 1 lst))       ;; linenum
                                 'ret)))
                    (delq nil (copy-sequence compilation-error-list))))

        ;; emacs22,emacs23,emacs24

        ;; in emacs23 must force fontification to have the properties/markers
        ;; etc
        (font-lock-fontify-buffer)
	(let (locus)
	  (while (setq locus (my-compilation-next-error))
	    (my-push (list (my-line-number)
                           (my-compilation-locus-filename locus)
                           (my-compilation-locus-linenum locus))
                     'ret)))))

    (nreverse ret)))

(defun my-compilation-locus-filename (locus)
  (if (and (fboundp 'compilation--message-p)
           (compilation--message-p locus))
      (setq locus (compilation--message->loc locus)))
  (if (fboundp 'compilation--loc->file-struct)
      (car ;; filename
       (compilation--file-struct->file-spec
        (compilation--loc->file-struct locus)))
    (caar (nth 2 (car locus)))))
(defun my-compilation-locus-linenum (locus)
  (if (and (fboundp 'compilation--message-p)
           (compilation--message-p locus))
      (setq locus (compilation--message->loc locus)))
  (if (fboundp 'compilation--loc->line)
      (compilation--loc->line locus)
    (nth 1 (car locus))))

(defun my-compilation-plain-list-p (obj)
  "Return non-nil if OBJ is a plain list, possibly empty.
This means non-circular and with a final nil tail, not a dotted end."
  (or (null obj)
      (and (consp obj)
           (let ((len (safe-length obj)))
             (and (natnump len)
                  (eq nil (nthcdr len obj)))))))

;; check ...
(mapcar (lambda (elem)
          (let* ((want (nth 0 elem))
                (obj  (nth 1 elem))
                (got  (my-compilation-plain-list-p obj)))
            (unless (equal got want)
              (error "my-compilation-plain-list-p: %S got %S want %S"
                     obj got want))))
        `((t nil)
          (t (1))
          (nil (1 . 2))
          (nil (1 2 . 3))
          (nil [])
          (nil [1])
          (nil 1)
          (nil ,(let ((circ (list 1)))
                  (setcdr circ circ)
                  circ))
          (nil ,(let ((circ (list 1 2)))
                  (setcdr (cdr circ) circ)
                  circ))))

(defun my-compilation-list-of-pred-p (lst pred)
  "Return non-nil if all elements of LST pass function PRED."
  (not (memq nil (mapcar pred lst))))

(defun my-compilation-natnum-or-null-p (obj)
  "Return non-nil if OBJ is either a `natnump' or nil."
  (or (natnump obj)
      (null obj)))

(defun my-compilation-validate-match (elem num)
  (unless (or (my-compilation-natnum-or-null-p num)
              (and (consp num)
                   (my-compilation-natnum-or-null-p (car num))
                   (my-compilation-natnum-or-null-p (cdr num))))
    (error "bad compilation-alist match num %S in\n%S" num elem)))

(defun my-compilation-validate-file (elem num)
  (unless (or (null num)
              (functionp num))
    (if (and (my-compilation-plain-list-p num)
             (my-compilation-list-of-pred-p (cdr num) 'stringp))
        (setq num (car num)))
    (my-compilation-validate-match elem num)))

(defun my-compilation-validate-part (elem part)
  (unless (and (my-compilation-plain-list-p part)
               (>= (length part) 2)
               (stringp (nth 0 part)))
    (error "bad compilation-alist part %S in\n%S" part elem))

  (my-compilation-validate-file elem (nth 1 part))
  (my-compilation-validate-match elem (nth 2 part)) ;; line
  (my-compilation-validate-match elem (nth 3 part)) ;; column
  (my-compilation-validate-match elem (nth 4 part)) ;; type
  (my-compilation-validate-match elem (nth 5 part)) ;; hyperlink
  ;; font-lock highlights from 6 onwards ...
  )

(my-compilation-validate-part
 nil '("regexp" nil 1 nil 2 0 (2 (compilation-face '(3)))))

(defun my-compilation-validate-alists ()
  "Check entries in `compilation-error-regexp-alist' and friends are valid."
  (require 'compile)

  (cond
   ;; xemacs21 -- 
   ((fboundp 'compilation-build-compilation-error-regexp-alist)

    ;; regexp-alist
    (mapcar (lambda (part)
              (my-compilation-validate-part part part))
            compilation-error-regexp-alist)

    ;; regexp-alist-alist
    ;; entries like (4bsd PART PART ...) or ((4bsd mips) PART PART...)
    (mapcar (lambda (elem)
              (unless (and (my-compilation-plain-list-p elem)
                           (>= (length elem) 2)
                           (or (symbolp (car elem))
                               (my-compilation-list-of-pred-p (car elem)
                                                              'symbolp)))
                (error "bad compilation-error-regexp-alist-alist element\n%S"
                       elem))
              (mapcar (lambda (part)
                        (my-compilation-validate-part elem part))
                      (cdr elem)))
            compilation-error-regexp-alist-alist))

   ;; emacs22
   ((boundp 'compilation-error-regexp-alist-alist)

    ;; regexp-alist, can be a symbol too
    (mapcar (lambda (elem)
              (unless (symbolp elem)
                (my-compilation-validate-part elem elem)))
            compilation-error-regexp-alist)

    ;; regexp-alist-alist, entries like (symbol . PART)
    (mapcar (lambda (elem)
              (unless (and (my-compilation-plain-list-p elem)
                           (symbolp (car elem)))
                (error "bad compilation-error-regexp-alist-alist element\n%S"
                       elem))
              (my-compilation-validate-part elem (cdr elem)))
            compilation-error-regexp-alist-alist))

   ;; emacs21
   (t
    ;; regexp-alist
    (mapcar (lambda (part)
              (my-compilation-validate-part part part))
            compilation-error-regexp-alist))))

(provide 'my-compile-testing)
