;; Copyright 2009, 2010, 2014 Kevin Ryde

;; t-commentary.el is shared by several distributions.
;;
;; t-commentary.el is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; t-commentary.el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with this file.  If not, see <http://www.gnu.org/licenses/>.


(require 'finder)
(require 'lisp-mnt)
(eval-when-compile
  (unless (fboundp 'dolist)
    (require 'cl))) ;; for `dolist' in emacs20

(defvar finder-known-keywords-list
  (append '("convenience" "files" "multimedia")
          (mapcar (lambda (pair)
                    (symbol-name (car pair)))
                  finder-known-keywords))
  "List of keywords from finder-known-keywords, as strings.
\"convenience\", \"files\" and \"multimedia\" are added to the
list, as they're not otherwise in xemacs21.")

(defun my-run-test ()
  (let* ((count 0)

         (srcdir (expand-file-name ".."))
         (file-list (directory-files srcdir nil "\\`[^.].*\\.el\\'"))

         (load-path (cons srcdir load-path))
         (find-function-source-path nil))

    (message "srcdir is %S" srcdir)
    (when (null file-list)
      (error "Oops, no *.el files"))

    ;; xemacs 21.4.22 in -batch mode segfaults in `finder-commentary' on
    ;; something to do with window size, so dummy out this shrink
    (defun shrink-window-if-larger-than-buffer (&optional window))

    (dolist (basename file-list)
      (let ((fullname (expand-file-name basename srcdir)))
        (message "check %S" basename)

        ;;-------------------------------------------------------------------
        ;; finder-commentary

        (unless (member basename '("miscbits-el-finder-inf.el"))
          (finder-commentary fullname)
          (setq count (1+ count))
          (kill-buffer nil))

        ;;-------------------------------------------------------------------
        ;; lm-authors

        (find-file fullname)

        (unless (member basename '("mine-sweeper-auto.el"))
          (let ((got (lm-authors)))
            (when (null got)
              (error "lm-authors %S got %S want list" fullname got)))
          (setq count (1+ count)))

        ;;-------------------------------------------------------------------
        ;; lm-code-mark
        ;; (aliased to lm-code-start in emacs22 up)

        (unless (member basename '("mine-sweeper-auto.el"))
          (let ((got (lm-code-mark)))
            (unless (numberp got)
              (error "lm-code-mark %S got %S want number" fullname got)))
          (setq count (1+ count)))

        ;;-------------------------------------------------------------------
        ;; lm-get-package-name

        (let ((got  (lm-get-package-name))
              (want basename))
          (unless (equal want got)
            (error "lm-get-package-name %S got %S want %S"
                   fullname got want)))
        (setq count (1+ count))

        ;;-------------------------------------------------------------------
        ;; lm-keywords
        ;; (no lm-keywords-list in xemacs21)

        (let* ((keyword-re "[a-z][-a-z0-9]*")
               (want-re    (concat "\\`"
                                   keyword-re
                                   "\\(, ?" keyword-re "\\)*"
                                   "\\'"))
               (got-str    (lm-keywords))
               (got-list   (and (stringp got-str)
                                ;; per emacs22 `lm-keywords-list'
                                (split-string got-str ",?[ \t]"))))

          ;; keywords present
          (unless (member basename '("mine-sweeper-auto.el"))
            (unless (and (stringp got-str)
                         (string-match want-re got-str))
              (error "lm-keywords %S got %S want-re %S"
                     fullname got-str want-re))
            (setq count (1+ count)))

          ;; at least one keyword known
          (unless (member basename '("mine-sweeper-auto.el"
                                     "miscbits-generate-finder.el"
                                     "miscbits-generate-loaddefs.el"
                                     "miscbits-el-finder-inf.el"
                                     "miscbits-el-loaddefs.el"))
            (let (found)
              (dolist (keyword got-list)
                (when (member keyword finder-known-keywords-list)
                  (setq found t)))
              (unless found
                (error "lm-keywords %S got %S not in finder-known-keywords: %S"
                       fullname got-str finder-known-keywords-list)))
            (setq count (1+ count))))

        ;;---------------------------------------------------------------------
        ;; lm-version

        (unless (member basename '("mine-sweeper-auto.el"
                                   "miscbits-byte-compile.el"
                                   "miscbits-generate-finder.el"
                                   "miscbits-generate-loaddefs.el"
                                   "miscbits-el-loaddefs.el"
                                   "miscbits-el-finder-inf.el"))
          (let ((header-version (lm-version))
                history-version)

            ;; version 0 rejected
            (let ((want-re "\\`[1-9][0-9]*\\'"))
              (unless (and (stringp header-version)
                           (string-match want-re header-version))
                (error "lm-version %S got %S want-re %S"
                       fullname header-version want-re))
              (setq count (1+ count)))

            (goto-char (point-min))
            (while (re-search-forward "^;; Version \\([0-9]+\\) - " nil t)
              (let ((v (match-string 1)))
                (when (or (not history-version)
                          (> (string-to-number v)
                             (string-to-number history-version)))
                  (setq history-version v))))
            (unless (equal history-version header-version)
              (error "%s header version %S history version %S"
                     fullname header-version history-version))
            (setq count (1+ count))))

        ;;---------------------------------------------------------------------
        ;; lm-header "URL"

        (unless (member basename '("mine-sweeper-auto.el"))
          (let ((want-re "\\`http://"))
            (let ((got  (lm-header "URL")))
              (unless (and (stringp got)
                           (string-match want-re got))
                (error "lm-header URL %S got %S want %S"
                       fullname got want-re))))
          (setq count (1+ count)))

        ;; the find-file
        (kill-buffer nil)))

    (message "  %d tests" count)
    count))
